// Generated by CoffeeScript 1.11.1
(function() {
  var Field, ForwardIndex, InvertedIndex, SortedSet, SparseVector, TextIndex, TokenPipeline, Tokenizer;

  SortedSet = require('./SortedSet');

  ForwardIndex = require('./ForwardIndex');

  InvertedIndex = require('./InvertedIndex');

  SparseVector = require('./SparseVector');

  Tokenizer = require('./Tokenizer');

  TokenPipeline = require('./TokenPipeline');

  Field = require('./Field');

  module.exports = TextIndex = (function() {
    function TextIndex(spec) {
      var fspec, k, ref;
      this.spec = spec;
      this.fields = {};
      this.nFields = 0;
      ref = this.spec.fields;
      for (k in ref) {
        fspec = ref[k];
        this.fields[k] = new Field(this, fspec, k);
        this.nFields++;
      }
      this.documentStore = new ForwardIndex;
      this.tokenStore = new InvertedIndex;
      this.corpusTokens = new SortedSet;
      this.tokenizer = this.spec.tokenizer || new Tokenizer;
      this.pipeline = this.spec.pipeline || new TokenPipeline;
      this._idfCache = {};
    }

    TextIndex.prototype.identify = function(doc) {
      return doc.id;
    };

    TextIndex.prototype.add = function(doc) {
      var allDocumentTokens, docRef, docTokens, field, fieldToken, fieldTokens, i, j, k, l, len, len1, len2, ref, ref1, ref2, tf, token, tokenCount;
      docRef = this.identify(doc);
      docTokens = {};
      allDocumentTokens = new SortedSet;
      ref = this.fields;
      for (k in ref) {
        field = ref[k];
        fieldTokens = this.pipeline.run(this.tokenizer.run(field.get(doc)));
        docTokens[field.name] = fieldTokens;
        for (i = 0, len = fieldTokens.length; i < len; i++) {
          token = fieldTokens[i];
          allDocumentTokens.add(token);
          this.corpusTokens.add(token);
        }
      }
      this.documentStore.set(docRef, allDocumentTokens);
      ref1 = allDocumentTokens.elements;
      for (j = 0, len1 = ref1.length; j < len1; j++) {
        token = ref1[j];
        tf = 0;
        ref2 = this.fields;
        for (k in ref2) {
          field = ref2[k];
          fieldTokens = docTokens[field.name];
          if (fieldTokens.length === 0) {
            continue;
          }
          tokenCount = 0;
          for (l = 0, len2 = fieldTokens.length; l < len2; l++) {
            fieldToken = fieldTokens[l];
            if (fieldToken === token) {
              tokenCount++;
            }
          }
          tf += tokenCount / fieldTokens.length * field.boost;
        }
        this.tokenStore.add(token, docRef, {
          id: docRef,
          tf: tf
        });
      }
      this._idfCache = {};
      return void 0;
    };

    TextIndex.prototype.removeById = function(docRef) {
      var docTokens, i, len, token;
      if (!this.documentStore.has(docRef)) {
        return;
      }
      docTokens = this.documentStore.get(docRef);
      this.documentStore.remove(docRef);
      for (i = 0, len = docTokens.length; i < len; i++) {
        token = docTokens[i];
        this.tokenStore.remove(token, docRef);
      }
      return void 0;
    };

    TextIndex.prototype.remove = function(doc) {
      return this.removeById(this.identify(doc));
    };

    TextIndex.prototype.update = function(doc) {
      this.remove(doc);
      return this.add(doc);
    };

    TextIndex.prototype.idf = function(term) {
      var cacheKey, df, idf;
      cacheKey = '@' + term;
      if (cacheKey in this._idfCache) {
        return this._idfCache[cacheKey];
      }
      df = this.tokenStore.count(term);
      idf = 1;
      if (df > 0) {
        idf = 1 + Math.log(this.documentStore.length / df);
      }
      return this._idfCache[cacheKey] = idf;
    };

    TextIndex.prototype.documentVector = function(id) {
      var i, idf, len, ref, tf, token, tokens, vector;
      tokens = this.documentStore.get(id);
      vector = new SparseVector;
      ref = tokens.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        token = ref[i];
        tf = this.tokenStore.get(token)[id].tf;
        idf = this.idf(token);
        vector.insert(this.corpusTokens.indexOf(token), tf * idf);
      }
      return vector;
    };

    TextIndex.prototype.search = function(query, operator) {
      var accumulator, diff, documentSets, expansion, expansions, field, fieldBoosts, i, id, idf, ids, j, k, l, len, len1, len2, matchingDocuments, pos, queryVector, reducer, ref, similarityBoost, tf, token, tokens;
      if (operator == null) {
        operator = "AND";
      }
      tokens = this.pipeline.run(this.tokenizer.run(query));
      queryVector = new SparseVector;
      documentSets = [];
      fieldBoosts = 0;
      ref = this.fields;
      for (k in ref) {
        field = ref[k];
        fieldBoosts += field.boost;
      }
      if (!tokens.some((function(_this) {
        return function(token) {
          return _this.tokenStore.has(token);
        };
      })(this))) {
        return [];
      }
      for (i = 0, len = tokens.length; i < len; i++) {
        token = tokens[i];
        tf = 1 / (tokens.length * this.nFields * fieldBoosts);
        accumulator = new SortedSet;
        expansions = this.tokenStore.expand(token);
        for (j = 0, len1 = expansions.length; j < len1; j++) {
          expansion = expansions[j];
          similarityBoost = 1;
          if (expansion !== token) {
            diff = Math.max(3, expansion.length - token.length);
            similarityBoost = 1 / Math.log(diff);
          }
          pos = this.corpusTokens.indexOf(expansion);
          idf = this.idf(expansion);
          if (pos > -1) {
            queryVector.insert(pos, tf * idf * similarityBoost);
          }
          matchingDocuments = this.tokenStore.get(expansion);
          ids = Object.keys(matchingDocuments);
          for (l = 0, len2 = ids.length; l < len2; l++) {
            id = ids[l];
            accumulator.add(matchingDocuments[id].id);
          }
        }
        documentSets.push(accumulator);
      }
      reducer = operator === 'AND' ? function(memo, set) {
        return memo.intersect(set);
      } : function(memo, set) {
        return memo.union(set);
      };
      return documentSets.reduce(reducer).map((function(_this) {
        return function(id) {
          return {
            id: id,
            score: queryVector.similarity(_this.documentVector(id))
          };
        };
      })(this));
    };

    return TextIndex;

  })();

}).call(this);
